package de.w665.sharepulse.db;

import com.rethinkdb.RethinkDB;
import com.rethinkdb.gen.exc.ReqlOpFailedError;
import com.rethinkdb.net.Connection;
import com.rethinkdb.net.Result;
import jakarta.annotation.PostConstruct;
import jakarta.annotation.PreDestroy;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.atomic.AtomicReference;

@Slf4j
@Service
public class RethinkDBService {

    private final RethinkDBConfig config;

    private final RethinkDB r;
    private final Connection connection;

    @Autowired
    public RethinkDBService(RethinkDBConfig config, RethinkDBConnector connector) {
        this.config = config;


        // mapping to private vars for easier access
        this.r = connector.getR();
        this.connection = connector.getConnection();
    }

    @PostConstruct
    public void initialize() {

        //rethinkdb check if database exists

        /*Result<Object> result = r.dbList().run(connection);
        List<String> databases = new ArrayList<>();

        Object db = result.first();
        ArrayList<String> dbList = (ArrayList<String>) db;
        databases.addAll(dbList);

        if (!databases.contains("sharepulse")) {
            r.dbCreate("sharepulse").run(connection);
            log.info("Database 'sharepulse' created successfully.");
        } else {
            log.info("Database 'sharepulse' already exists. No action needed.");
        }*/

        // rethinkdb check if database exists
        try {
            r.dbCreate(config.getDatabase()).run(connection).stream();
            log.debug("Database " + config.getDatabase() + " created");
        } catch (ReqlOpFailedError e) {
            log.debug("Database " + config.getDatabase() + " already exists. Error: " + e.getClass().getSimpleName());
        }

        // rethinkdb check if table file_uploads exists
        try {
            r.db(config.getDatabase()).tableCreate("file_uploads").run(connection).stream();
            log.debug("Table 'file_uploads' created successfully.");
        } catch (ReqlOpFailedError e) {
            log.debug("Table 'file_uploads' already exists. Clearing content...");
            r.db(config.getDatabase()).table("file_uploads").delete().run(connection);
            log.debug("Table 'file_uploads' cleared successfully.");
        }

        // rethinkdb check if table id_store exists
        try {
            r.db(config.getDatabase()).tableCreate("id_store").run(connection).stream();
            log.debug("Table 'id_store' created successfully.");
        } catch (ReqlOpFailedError e) {
            log.debug("Table 'id_store' already exists. No action needed.");
        }
        log.info("Database ready for operation!");
    }

    @PreDestroy
    public void close() {
        if (connection != null) {
            connection.close();
        }
    }
}
